export function extract_links(headers) {
    const link = headers.get("link");
    if (!link)
        return [];
    const out = [];
    for (let l of link.split(",")) {
        let [key, val] = l.split(";");
        if (!key || !val)
            continue;
        const url = key.trim().slice(1, -1);
        const [val_key, val_val] = val.split("=");
        if (!val_key || !val_val)
            continue;
        if (val_key.trim().toLowerCase() !== "rel")
            continue;
        const target = val_val.trim().slice(1, -1);
        out.push({ target, url });
    }
    return out;
}
export function findNextUrl(resp) {
    const startingUrl = new URL(resp.url);
    const links = extract_links(resp.headers);
    const next = links.find((x) => x.target === "next");
    if (next) {
        const url_url = new URL(next.url, `${startingUrl.protocol}//${startingUrl.host}`);
        return url_url.href;
    }
}
import winston, { format } from "winston";
const PROCESSOR_NAME = "omeka-s";
const consoleTransport = new winston.transports.Console();
consoleTransport.level =
    process.env.LOG_LEVEL ||
        process.env.DEBUG?.includes(PROCESSOR_NAME) ||
        process.env.DEBUG === "*"
        ? "debug"
        : "info";
const classLoggers = new WeakMap();
export function getLoggerFor(loggable) {
    let logger;
    if (typeof loggable === "string") {
        logger = createLogger(loggable);
    }
    else {
        const { constructor } = loggable;
        if (classLoggers.has(loggable)) {
            logger = classLoggers.get(loggable);
        }
        else {
            logger = createLogger(constructor.name);
            classLoggers.set(loggable, logger);
        }
    }
    return logger;
}
function createLogger(label) {
    return winston.createLogger({
        format: format.combine(format.label({ label }), format.colorize(), format.timestamp(), format.metadata({
            fillExcept: ["level", "timestamp", "label", "message"],
        }), format.printf(({ level: levelInner, message, label: labelInner, timestamp, }) => `${timestamp} {${PROCESSOR_NAME}} [${labelInner}] ${levelInner}: ${message}`)),
        transports: [consoleTransport],
    });
}
